<?php


class Contently_Coauthors {

    private $coauthor_taxonomy = 'author';

    /**
     * Get the author term for a given co-author
     *
     * @param object $coauthor The co-author object
     * @return object|false $author_term The author term on success
     */
    public function get_author_term( $coauthor ) {

        if ( ! is_object( $coauthor ) ) {
            return false;
        }

        $cache_key = 'author-term-' . $coauthor->user_nicename;
        if ( false !== ( $term = wp_cache_get( $cache_key, 'co-authors-plus' ) ) ) {
            return $term;
        }

        // See if the prefixed term is available, otherwise default to just the nicename
        $term = get_term_by( 'slug', 'cap-' . $coauthor->user_nicename, $this->coauthor_taxonomy );
        if ( ! $term ) {
            $term = get_term_by( 'slug', $coauthor->user_nicename, $this->coauthor_taxonomy );
        }
        wp_cache_set( $cache_key, $term, 'co-authors-plus' );
        return $term;
    }

    /**
     * Update the author term for a given co-author
     *
     * @param object $coauthor The co-author object (user or guest author)
     * @return object|false $success Term object if successful, false if not
     */
    public function update_author_term( $coauthor ) {

        if ( ! is_object( $coauthor ) ) {
            return false;
        }

        // Update the taxonomy term to include details about the user for searching
        $search_values = array();
        foreach ( ['display_name', 'first_name', 'last_name', 'user_login', 'ID', 'user_email'] as $search_field ) {
            $search_values[] = $coauthor->$search_field;
        }
        $term_description = implode( ' ', $search_values );

        if ( $term = $this->get_author_term( $coauthor ) ) {
            if ( $term->description != $term_description ) {
                wp_update_term( $term->term_id, $this->coauthor_taxonomy, array( 'description' => $term_description ) );
            }
        } else {
            $coauthor_slug = 'cap-' . $coauthor->user_nicename;
            $args = array(
                'slug'          => $coauthor_slug,
                'description'   => $term_description,
            );

            wp_insert_term( $coauthor->user_login, $this->coauthor_taxonomy, $args );
        }
        wp_cache_delete( 'author-term-' . $coauthor->user_nicename, 'co-authors-plus' );
        return $this->get_author_term( $coauthor );
    }

    /**
     * Add one or more co-authors as bylines for a post
     *
     * @param int
     * @param array
     *
     * @return boolean
     */
    public function add_coauthors( $post_id, $coauthors ) {
        $post_id = (int) $post_id;

        $coauthor_objects = [];
        foreach ( $coauthors as &$author_name ) {
            $author = get_user_by( 'slug', $author_name );

            if (!$author) continue;

            $coauthor_objects[] = $author;
            $term = $this->update_author_term( $author );
            if ( is_object( $term ) ) {
                $author_name = $term->slug;
            }
        }
        wp_set_post_terms( $post_id, $coauthors, $this->coauthor_taxonomy, false );
        return true;

    }
}